@@ -6,8 +6,13 @@
 
 set -euxo pipefail
 
-# Make sure we're on latest, mostly for Clippy's sake
-rustup update
+# Make sure we're on latest, mostly for Clippy's sake. On CI, we don't do this
+# because GitHub already put some level of this tooling in place.
+#
+# $CI check from: https://stackoverflow.com/a/13864829/473672
+if [[ -z "${CI+x}" ]]; then
+    rustup update
+fi
 rustup component add clippy rustfmt
 
 # Settings are at the top of main.rs
@@ -18,6 +23,14 @@ cargo clippy
 cargo build --workspace
 cargo test --workspace
 
+if [[ -z "${CI+x}" ]]; then
+    # Try a Windows build, cross compiles must work
+    #
+    # Only locally, on CI this should be covered by windows-ci.yml.
+    rustup target add x86_64-pc-windows-gnu
+    cargo build --release --target=x86_64-pc-windows-gnu
+fi
+
 # If you have an editor that formats on save this will never be a problem
 cargo fmt -- --check
 
