@@ -1,5 +1,11 @@
+use crate::ansi::remove_ansi_escape_codes;
 use crate::commit_line::format_commit_line;
 use crate::io::ErrorKind;
+use crate::refiner::to_highlighted_tokens;
+use crate::token_collector::{
+    lowlight_timestamp, render, unhighlight_git_prefix, LINE_STYLE_NEW_FILENAME,
+    LINE_STYLE_OLD_FILENAME,
+};
 use std::io::{self, BufWriter, Write};
 use std::process::exit;
 use std::sync::mpsc::{sync_channel, Receiver, SyncSender};
@@ -7,7 +13,6 @@ use std::sync::{Arc, Mutex};
 use std::thread::{self, JoinHandle};
 
 use crate::{constants::*, refiner};
-use regex::Regex;
 use threadpool::ThreadPool;
 
 const HUNK_HEADER: &str = "\x1b[36m"; // Cyan
@@ -24,12 +29,8 @@ lazy_static! {
         ("similarity index ", FAINT),
         ("new file mode ", FAINT),
         ("deleted file mode ", FAINT),
-        ("--- /dev/null", FAINT),
-        ("+++ /dev/null", FAINT),
     ];
 
-    static ref ANSI_COLOR_REGEX: Regex = Regex::new("\x1b\\[[0-9;]*[^0-9;]").unwrap();
-
     /// This is the `\ No newline at end of file` string. But since it can come
     /// in not-English as well as English, we take it from the input and store it
     /// in this variable. None means we don't know yet.
@@ -306,21 +307,71 @@ impl LineCollector {
     }
 
     pub fn consume_plusminus_header(&mut self, line: &str) {
-        self.consume_plain_linepart(BOLD);
-
-        if let Some(last_tab_index) = line.rfind('\t') {
-            self.consume_plain_linepart(&line[..last_tab_index]);
-
-            // When I ran plain "diff" (no git involved), this trailing part
-            // contained very precise file timestamps. I don't think those
-            // provide much value, so let's faint them out.
-            self.consume_plain_linepart(FAINT);
-            self.consume_plain_linepart(&line[last_tab_index..]);
-        } else {
-            self.consume_plain_linepart(line);
+        if let Some(old_name) = line.strip_prefix("--- ") {
+            self.old_text.clear();
+            self.old_text.push_str(old_name);
+            return;
         }
 
-        self.consume_plain_line(NORMAL);
+        if let Some(new_name) = line.strip_prefix("+++ ") {
+            if self.old_text.is_empty() {
+                // We got +++ not preceded by ---, WTF?
+                return;
+            }
+
+            self.new_text.clear();
+            self.new_text.push_str(new_name);
+        } else {
+            panic!("Got a plusminus header that doesn't start with --- or +++");
+        }
+
+        if self.old_text == "/dev/null" {
+            let new_name = self.new_text.clone();
+            self.old_text.clear();
+            self.new_text.clear();
+
+            self.consume_plain_linepart(FAINT);
+            self.consume_plain_linepart("--- /dev/null");
+            self.consume_plain_line(NORMAL);
+
+            self.consume_plain_linepart(BOLD);
+            self.consume_plain_linepart("+++ ");
+            self.consume_plain_linepart(&new_name);
+            self.consume_plain_line(NORMAL);
+            return;
+        }
+
+        if self.new_text == "/dev/null" {
+            let old_name = self.old_text.clone();
+            self.old_text.clear();
+            self.new_text.clear();
+
+            self.consume_plain_linepart(BOLD);
+            self.consume_plain_linepart("--- ");
+            self.consume_plain_linepart(&old_name);
+            self.consume_plain_line(NORMAL);
+
+            self.consume_plain_linepart(FAINT);
+            self.consume_plain_linepart("+++ /dev/null");
+            self.consume_plain_line(NORMAL);
+
+            return;
+        }
+
+        let (mut old_tokens, mut new_tokens, _, _) =
+            to_highlighted_tokens(&self.old_text, &self.new_text);
+        self.old_text.clear();
+        self.new_text.clear();
+
+        lowlight_timestamp(&mut old_tokens);
+        unhighlight_git_prefix(&mut old_tokens);
+        lowlight_timestamp(&mut new_tokens);
+        unhighlight_git_prefix(&mut new_tokens);
+
+        let old_filename = render(&LINE_STYLE_OLD_FILENAME, old_tokens);
+        let new_filename = render(&LINE_STYLE_NEW_FILENAME, new_tokens);
+        self.consume_plain_line(&old_filename);
+        self.consume_plain_line(&new_filename);
     }
 
     fn consume_hunk_header(&mut self, line: &str) {
@@ -339,15 +390,12 @@ impl LineCollector {
         self.consume_plain_line(NORMAL);
     }
 
-    fn without_ansi_escape_codes(input: &'_ str) -> std::borrow::Cow<'_, str> {
-        return ANSI_COLOR_REGEX.replace_all(input, "");
-    }
-
     /// The line parameter is expected *not* to end in a newline
-    pub fn consume_line(&mut self, line: String) {
+    pub fn consume_line(&mut self, line: &mut Vec<u8>) {
         // Strip out incoming ANSI formatting. This enables us to highlight
         // already-colored input.
-        let line = LineCollector::without_ansi_escape_codes(&line);
+        remove_ansi_escape_codes(line);
+        let line = String::from_utf8_lossy(line);
 
         if line.starts_with("diff") {
             self.diff_seen = true;
@@ -416,35 +464,3 @@ impl LineCollector {
         self.consume_plain_line(&line);
     }
 }
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-
-    #[cfg(test)]
-    use pretty_assertions::assert_eq;
-
-    #[test]
-    fn test_non_sgr() {
-        assert_eq!(
-            LineCollector::without_ansi_escape_codes("hel\x1b[0Klo"),
-            "hello"
-        );
-    }
-
-    #[test]
-    fn test_sgr() {
-        assert_eq!(
-            LineCollector::without_ansi_escape_codes("hel\x1b[33mlo"),
-            "hello"
-        );
-    }
-
-    #[test]
-    fn test_multi_sgr() {
-        assert_eq!(
-            LineCollector::without_ansi_escape_codes("hel\x1b[33;34mlo"),
-            "hello"
-        );
-    }
-}
