@@ -9,11 +9,13 @@
 extern crate lazy_static;
 
 use backtrace::Backtrace;
+use clap::CommandFactory;
+use clap::Parser;
 use git_version::git_version;
 use line_collector::LineCollector;
 use std::io::{self, IsTerminal};
 use std::panic;
-use std::path;
+use std::path::{self, PathBuf};
 use std::process::exit;
 use std::process::{Command, Stdio};
 use std::str;
@@ -27,29 +29,15 @@ mod refiner;
 mod token_collector;
 mod tokenizer;
 
-const HELP_TEXT: &str = r#"
-Usage:
-  diff ... | riff
-  riff [-b] [--no-pager] <file1> <file2>
-  riff [-b] [--no-pager] <directory1> <directory2>
-
-Colors diff output, highlighting the changed parts of every line.
+const HELP_TEXT_FOOTER: &str = r#"Installing riff in the $PATH:
+  sudo cp riff /usr/local/bin
 
 Git integration:
-    git config --global pager.diff riff
-    git config --global pager.show riff
-    git config --global pager.log riff
-    git config --global interactive.diffFilter riff
+  git config --global pager.diff riff
+  git config --global pager.show riff
+  git config --global pager.log riff
+  git config --global interactive.diffFilter riff
 
-Options:
-    -b:         Ignore changes in amount of whitespace
-    --no-pager: Don't page the result
-
-    --help:     Print this text
-    --version:  Print version number
-"#;
-
-const HELP_TEXT_FOOTER: &str = r#"
 Report issues at <https://github.com/walles/riff>.
 "#;
 
@@ -67,6 +55,43 @@ const PAGER_FORKBOMB_STOP: &str = "_RIFF_IGNORE_PAGER";
 // Ref: https://github.com/walles/riff/issues/26#issuecomment-1120294897
 const GIT_VERSION: &str = git_version!(cargo_prefix = "");
 
+#[derive(Parser)]
+#[command(
+    version = GIT_VERSION,
+    name = "riff",
+    about = "Colors diff output, highlighting the changed parts of every line.",
+    after_help = HELP_TEXT_FOOTER,
+    override_usage = r#"
+  diff ... | riff [--no-pager]
+  riff [-b] [--no-pager] <C1> <C2>
+  riff [-b] [--no-pager] --file <FILE>"#
+)]
+
+struct Options {
+    /// First file or directory to compare
+    #[arg(requires("c2"))]
+    c1: Option<String>,
+
+    /// Second file or directory to compare
+    #[arg()]
+    c2: Option<String>,
+
+    /// Read diff or patch file
+    #[arg(long, short, conflicts_with_all = ["c1", "c2"])]
+    file: Option<PathBuf>,
+
+    /// Ignore changes in amount of whitespace
+    #[arg(short('b'), requires("c1"))]
+    ignore_space_change: bool,
+
+    /// Don't page the result
+    #[arg(long)]
+    no_pager: bool,
+
+    #[arg(long, hide(true))]
+    please_panic: bool,
+}
+
 fn highlight_diff<W: io::Write + Send + 'static>(input: &mut dyn io::Read, output: W) {
     let mut line_collector = LineCollector::new(output);
 
@@ -153,40 +178,6 @@ fn try_pager(input: &mut dyn io::Read, pager_name: &str) -> bool {
     }
 }
 
-/// If `option` is found in `argv`, all instances of `option` will be removed
-/// from `argv`.
-///
-/// Returns `true` if `option` was found and consumed, false otherwise.
-#[must_use]
-fn consume(option: &str, argv: &mut Vec<String>) -> bool {
-    if !argv.contains(&option.to_string()) {
-        // Not found
-        return false;
-    }
-
-    argv.retain(|x| x != option);
-    return true;
-}
-
-fn print_help(output: &mut dyn io::Write) {
-    output.write_all(HELP_TEXT.trim().as_bytes()).unwrap();
-    output.write_all(b"\n").unwrap();
-    output.write_all(b"\n").unwrap();
-
-    output
-        .write_all(b"Installing riff in the $PATH:\n")
-        .unwrap();
-    output
-        .write_all(b"    sudo cp riff /usr/local/bin\n")
-        .unwrap();
-    output.write_all(b"\n").unwrap();
-
-    output
-        .write_all(HELP_TEXT_FOOTER.trim().as_bytes())
-        .unwrap();
-    output.write_all(b"\n").unwrap();
-}
-
 fn panic_handler(panic_info: &panic::PanicInfo) {
     eprintln!("\n\n-v-v-v----------- RIFF CRASHED ---------------v-v-v-\n",);
 
@@ -331,62 +322,52 @@ fn main() {
         panic_handler(panic_info);
     }));
 
-    let mut args: Vec<String> = env::args().collect();
-    if consume("--help", &mut args) || consume("-h", &mut args) {
-        print_help(&mut io::stdout());
-        return;
-    }
+    let options = Options::parse();
 
-    if consume("--version", &mut args) {
-        println!("riff {GIT_VERSION}");
-        println!();
-        println!("Source code available at <https://github.com/walles/riff>.");
-        return;
-    }
-
-    let ignore_space_change = consume("-b", &mut args);
-
-    if consume("--please-panic", &mut args) {
+    if options.please_panic {
         panic!("Panicking on purpose");
     }
 
-    let no_pager = consume("--no-pager", &mut args);
-
-    if args.len() == 3 {
+    if let (Some(file1), Some(file2)) = (options.c1, options.c2) {
         // "riff file1 file2"
         exec_diff_highlight(
-            args.get(1).unwrap(),
-            args.get(2).unwrap(),
-            ignore_space_change,
-            no_pager,
+            &file1,
+            &file2,
+            options.ignore_space_change,
+            options.no_pager,
         );
         return;
     }
 
-    if ignore_space_change {
-        eprintln!(
-            "ERROR: -b is only supported when diffing two named paths (\"riff -b a.txt b.txt\")"
-        );
-        eprintln!();
-        print_help(&mut io::stderr());
-        exit(1);
-    }
+    if let Some(diff_path) = options.file {
+        // riff -f file
+        if diff_path.is_dir() {
+            eprintln!("ERROR: --file cannot be a directory");
+            exit(1)
+        }
 
-    if args.len() != 1 {
-        eprintln!("ERROR: Unknown command line: {args:?}");
-        eprintln!();
-        print_help(&mut io::stderr());
-        exit(1);
+        let mut diff_file = match File::open(diff_path.clone()) {
+            Ok(f) => f,
+            Err(e) => {
+                eprintln!("ERROR: Can't open {}: {}", diff_path.to_string_lossy(), e);
+                exit(1);
+            }
+        };
+        highlight_stream(&mut diff_file, options.no_pager);
+        return;
     }
 
     if io::stdin().is_terminal() {
         eprintln!("ERROR: Expected input from a pipe");
         eprintln!();
-        print_help(&mut io::stderr());
+
+        // Print help to stderr
+        Options::command().write_help(&mut io::stderr()).unwrap();
+
         exit(1);
     }
 
-    highlight_stream(&mut io::stdin().lock(), no_pager);
+    highlight_stream(&mut io::stdin().lock(), options.no_pager);
 }
 
 #[cfg(test)]
